import pyi18next.i18next

RESOURCES = {
    'en': {
        'translation': {
            'key': 'value',
            'object': {"key": "value"},
            'array': ['value1', 'value2'],
            'interpolation': 'Hello, {{name}}!',
            'object_interpolation': {'greet': 'Hello, {{name}}!'},
            'array_interpolation': ['Hello', '{{name}}', '!'],
            'nested': '$t(key)',
            'nested_interpolation': '$t(interpolation, {"name": "Friend"})',
            'key_male': 'mail value',
            'key_female': 'female value',
            'keyPlural_one': 'one',
            'keyPlural_two': 'two',
            'keyPlural_other': 'other',
            'numberFormat': "Number: {{val, number(format: '0.00')}}",
            'datesFormat': "Datetime: {{val, datetime(format: 'yyyy-MM-dd')}}",
            'listFormat': "List: {{val, list(style: 'or')}}",
        },
        'special': { 
            'key': 'special value',
        }
    }
}

i18n = pyi18next.i18next.I18next(
            resources=RESOURCES,
            default_lng='en',
            default_ns='translation',
            plural_rules='one: n is 1; two: n is 2')

class TestTranslate:
    def test_base(self):
        assert i18n.t('key') == 'value'

    def test_deep(self):
        assert i18n.t('object.key') == 'value'

    def test_default(self):
        assert i18n.t('keyinexisted', 'default value') == 'default value'

    def test_cimode(self):
        assert i18n.t('key', lng='cimode') == 'key'
        assert i18n.t('key', lng='cimode', append_namespace_to_cimode=True) == 'translation:key'

    def test_special(self):
        assert i18n.t('key', ns='special') == 'special value'
        assert i18n.t('special:key') == 'special value'
    
    def test_object(self):
        assert i18n.t('object', return_objects=True) == {"key": "value"}
    
    def test_array(self):
        assert i18n.t('array', return_objects=True) == ['value1', 'value2']
        assert i18n.t('array', joinArrays=' ') == 'value1 value2'
    
    def test_interpolation(self):
        assert i18n.t('interpolation', name='world') == 'Hello, world!'
        assert i18n.t('object_interpolation', name='world', return_objects=True) == {'greet': 'Hello, world!'}
        assert i18n.t('array_interpolation', name='world', return_objects=True) == ['Hello', 'world', '!']
        assert i18n.t('array_interpolation', name='world', joinArrays=' ') == 'Hello world !'

    def test_nest(self):
        assert i18n.t('nested') == 'value'
        assert i18n.t('nested_interpolation') == 'Hello, Friend!'

    def test_context(self):
        assert i18n.t('key_male') == 'mail value'
        assert i18n.t('key_female') == 'female value'

    def test_plural(self):
        assert i18n.t('keyPlural', count=1) == 'one'
        assert i18n.t('keyPlural', count=2) == 'two'
        assert i18n.t('keyPlural', count=3) == 'other'

    def test_format(self):
        assert i18n.t('numberFormat', val=1024) == 'Number: 1024.00'
        import datetime
        assert i18n.t('datesFormat', val=datetime.datetime(2000,1,1)) == 'Datetime: 2000-01-01'
        assert i18n.t('listFormat', val=[1, 2, 3, 4]) == 'List: 1, 2, 3, or 4'


'''
Format functions for i18next.
'''
import babel.numbers
import babel.dates
import babel.lists


FUNCTIONS = {
    'number': babel.numbers.format_decimal,
    'currency': babel.numbers.format_currency,
    'format_compact_currency': babel.numbers.format_compact_currency,
    'format_compact_decimal': babel.numbers.format_compact_decimal,
    'format_currency': babel.numbers.format_currency,
    'format_decimal': babel.numbers.format_decimal,
    'format_number': babel.numbers.format_number,
    'format_percent': babel.numbers.format_percent,
    'format_scientific': babel.numbers.format_scientific,
    'datetime': babel.dates.format_datetime,
    'relativetime': babel.dates.format_timedelta,
    'format_date': babel.dates.format_date, 
    'format_datetime': babel.dates.format_datetime,
    'format_interval': babel.dates.format_interval,
    'format_skeleton': babel.dates.format_skeleton,
    'format_time': babel.dates.format_time,
    'format_timedelta': babel.dates.format_timedelta,
    'list': babel.lists.format_list,
    'format_list': babel.lists.format_list,
}


def parse_param_default_value(param_default_value):
    '''
    Parse a parameter default value in a format function.
    '''
    if param_default_value is None:
        return None
    if param_default_value.startswith("'") and param_default_value.endswith("'"):
        return param_default_value[1:-1]
    if param_default_value == 'true':
        return True
    if param_default_value == 'false':
        return False
    if '.' in param_default_value:
        return float(param_default_value)
    return int(param_default_value)


def apply_function(value, function_desc, **params):
    '''
    Apply a format function to a value.
    '''
    function_name, function_params = function_desc
    if function_name not in FUNCTIONS:
        raise ValueError(f'Unknown function: {function_name}')
    func = FUNCTIONS[function_name]
    func_annotations = getattr(func, '__annotations__', {})
    default_params = {}
    for param_name, param_default_value in function_params.items():
        if param_default_value is None:
            continue
        param_default_value = parse_param_default_value(param_default_value)
        if param_name in func_annotations:
            param_type = func_annotations[param_name]
            default_params[param_name] = param_type(param_default_value) if callable(param_type) else param_default_value
        else:
            default_params[param_name] = param_default_value
    call_params = {**default_params, **params}
    return func(value, **call_params)





import pyi18next.i18next
import pyi18next.backends.fs as fs

def test_class():
    i18n = pyi18next.i18next.I18next()
    assert i18n

def test_load():
    import json
    backend = fs.Backend(name_mapping=lambda lng, ns: f'tests/sample/{ns}/{lng}.json')
    i18n = pyi18next.i18next.I18next(lng='en', ns='translation', backend=backend)
    i18n.load()
    i18n.t('key', lng='en', ns='translation') == 'value'


"""
bitstillery_pyi18next - A Python implementation of i18next with pluralisation fixes.

This is a fork of pyi18next that fixes pluralisation count issues.
"""
__author__ = "Gongziting Tech Ltd., Bitstillery"

from .i18next import I18next

__all__ = ["I18next"]
